import { world, system } from "@minecraft/server"

const mapSystem = world.getDimension('overworld')
let placed = false;

system.afterEvents.scriptEventReceive.subscribe(data => {
    const { id, message } = data;
    if(id == "gst:piece_placed") {
        const blockList = [
        { x: 20, y: 122, z: 21 },
        { x: 20, y: 122, z: 22 },
        { x: 20, y: 122, z: 23 },
        { x: 20, y: 122, z: 24 },
        { x: 20, y: 122, z: 25 },
        { x: 20, y: 122, z: 26 },
        { x: 20, y: 122, z: 27 },
        { x: 20, y: 122, z: 28 },
        { x: 20, y: 122, z: 29 },
        { x: 20, y: 122, z: 30 }
        ]
        system.runTimeout(() => {
            const blocks = blockList.map(block => mapSystem.getBlock(block));
            if(blocks.some(element => !element.isAir)) {
                for(const player of world.getAllPlayers()) {
                    player.onScreenDisplay.setActionBar('§cGame Over!');
                    player.playSound('game_over.gst');
                    const score = world.scoreboard.getObjective('gst:score').getScore(player);
                    const highscore = world.scoreboard.getObjective('gst:highscore').getScore(player);
                    if(score > highscore) {
                        world.scoreboard.getObjective('gst:highscore').setScore(player, score);
                        world.sendMessage({'rawtext':[{'translate':'gst.new_score.message'},{'text':`§e${score}`}]});
                    } else {
                        world.sendMessage({'rawtext':[{'translate':'gst.score.message'},{'text':`§f${score}`}]});
                    }
                    system.runTimeout(() => {
                        player.setProperty('gst:start', false);
                    },5)
                }
                return;
            }
        },10);
        for(const player of world.getAllPlayers()) {
            const time = player.getProperty('gst:time');
            player.playSound('piece_placed.gst');
            player.runCommandAsync('function gst/check_full_lines');
            world.scoreboard.getObjective('gst:score').addScore(player, 1 + Math.floor(time*0.3));
            player.setProperty('gst:time', time == 30 ? time : 30);
            system.runTimeout(() => {
                if(placed) return;
                world.scoreboard.getObjective(`gst:${message}_piece`).addScore(player, 1);
                placed = true;
                system.runTimeout(() => {
                    placed = false;
                },10)
            },5);
        }
    }
});

world.afterEvents.entitySpawn.subscribe(({ entity }) => {
    for(const player of world.getAllPlayers()) {
        const type = player.getProperty('gst:type');
        const maxPieces = player.getProperty('gst:max_pieces');
        if(entity.matches({ families: ["piece"]})) {
            entity.runCommandAsync(`scoreboard players random @a gst:next_piece 1 ${maxPieces}`);
            entity.setProperty('gst:type', type);
        }
    }
})